@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -96,6 +96,7 @@ sub Create {
     my %args = (
         Name         => '',
         Summary      => '',
+        SortOrder    => 0,
         Class        => '0',
         CustomFields => {},
         Links        => {},
@@ -122,6 +123,7 @@ sub Create {
         Name    => $args{'Name'},
         Class   => $class->Id,
         Summary => $args{'Summary'},
+        SortOrder => $args{'SortOrder'},
         Disabled => $args{'Disabled'},
     );
     unless ($id) {
@@ -420,8 +422,8 @@ sub DeleteTopic {
         ObjectType => ref($self)
     );
     if ( $t->Id ) {
-        my $del = $t->Delete;
-        unless ($del) {
+        my ($ok, $msg) = $t->Delete;
+        unless ($ok) {
             return (
                 undef,
                 $self->loc(
@@ -541,6 +543,8 @@ the Article's Class is applied to that Queue.
 
 sub LoadByInclude {
     my $self = shift;
+    RT->Deprecated( Remove  => '5.2' );
+
     my %args = @_;
     my $Field = $args{Field};
     my $Value = $args{Value};
@@ -775,7 +779,7 @@ sub _CoreAccessible {
         Summary => 
                 {read => 1, write => 1, type => 'varchar(255)', default => ''},
         SortOrder => 
-                {read => 1, write => 1, type => 'int(11)', default => '0'},
+                {read => 1, write => 1, type => 'int(11)', default => '0', is_numeric => 1},
         Class => 
                 {read => 1, write => 1, type => 'int(11)', default => '0'},
         Parent => 
