@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -117,7 +117,6 @@ sub Create {
         Name        => '',
         Description => '',
         SortOrder   => '0',
-        HotList     => 0,
         @_
     );
 
@@ -135,7 +134,6 @@ sub Create {
         Name        => $args{'Name'},
         Description => $args{'Description'},
         SortOrder   => $args{'SortOrder'},
-        HotList     => $args{'HotList'},
     );
 
 }
@@ -474,24 +472,6 @@ Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
 =cut
 
 
-=head2 HotList
-
-Returns the current value of HotList. 
-(In the database, HotList is stored as int(2).)
-
-
-
-=head2 SetHotList VALUE
-
-
-Set HotList to VALUE. 
-Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
-(In the database, HotList will be stored as a int(2).)
-
-
-=cut
-
-
 =head2 Creator
 
 Returns the current value of Creator. 
@@ -542,8 +522,6 @@ sub _CoreAccessible {
                 {read => 1, write => 1, type => 'int(11)', default => '0'},
         Disabled => 
                 {read => 1, write => 1, type => 'int(2)', default => '0'},
-        HotList => 
-                {read => 1, write => 1, type => 'int(2)', default => '0'},
         Creator => 
                 {read => 1, auto => 1, type => 'int(11)', default => '0'},
         Created => 
