@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -61,6 +61,7 @@ use RT::Crypt::GnuPG::CRLFHandle;
 use GnuPG::Interface;
 use RT::EmailParser ();
 use RT::Util 'safe_run_child', 'mime_recommended_filename';
+use URI::Escape;
 
 =head1 NAME
 
@@ -173,7 +174,7 @@ Common options include:
 =item --homedir
 
 The GnuPG home directory where the keyrings are stored; by default it is
-set to F</opt/rt4/var/data/gpg>.
+set to F</opt/rt5/var/data/gpg>.
 
 You can manage this data with the 'gpg' commandline utility using the
 GNUPGHOME environment variable or C<--homedir> option.  Other utilities may
@@ -286,8 +287,6 @@ my %supported_opt = map { $_ => 1 } qw(
        verbose
 );
 
-our $RE_FILE_EXTENSIONS = qr/pgp|asc/i;
-
 # DEV WARNING: always pass all STD* handles to GnuPG interface even if we don't
 # need them, just pass 'IO::Handle->new()' and then close it after safe_run_child.
 # we don't want to leak anything into FCGI/Apache/MP handles, this break things.
@@ -329,6 +328,7 @@ sub CallGnuPG {
     my %GnuPGOptions = RT->Config->Get('GnuPGOptions');
     my %opt = (
         'digest-algo' => 'SHA1',
+        'cert-digest-algo' => 'SHA256',
         %GnuPGOptions,
         %{ $args{Options} || {} },
     );
@@ -824,12 +824,13 @@ sub FindScatteredParts {
         }
     }
 
+    my $file_extension_regex = join '|', @{ RT->Config->Get('GnuPG')->{FileExtensions} };
     # attachments with inline encryption
     foreach my $part ( @parts ) {
         next if $args{'Skip'}{$part};
 
         my $fname = $part->head->recommended_filename || '';
-        next unless $fname =~ /\.${RE_FILE_EXTENSIONS}$/;
+        next unless $fname =~ /\.(?:$file_extension_regex)$/;
 
         $RT::Logger->debug("Found encrypted attachment '$fname'");
 
@@ -848,7 +849,7 @@ sub FindScatteredParts {
         my $type = $self->_CheckIfProtectedInline( $part );
         next unless $type;
 
-        my $file = ($part->head->recommended_filename||'') =~ /\.${RE_FILE_EXTENSIONS}$/;
+        my $file = ($part->head->recommended_filename||'') =~ /\.(?:$file_extension_regex)$/;
 
         $args{'Skip'}{$part} = 1;
         push @res, {
@@ -1087,8 +1088,8 @@ sub DecryptInline {
 
             seek $block_fh, 0, 0;
 
-            my ($res_fh, $res_fn);
-            ($res_fh, $res_fn, %res) = $self->_DecryptInlineBlock(
+            my ($res_fh, $res_fn, $embedded_fn);
+            ($res_fh, $res_fn, $embedded_fn, %res) = $self->_DecryptInlineBlock(
                 %args,
                 BlockHandle => $block_fh,
             );
@@ -1123,8 +1124,8 @@ sub DecryptInline {
         # decrypt what we have, it can be just missing -----END PGP...
         seek $block_fh, 0, 0;
 
-        my ($res_fh, $res_fn);
-        ($res_fh, $res_fn, %res) = $self->_DecryptInlineBlock(
+        my ($res_fh, $res_fn, $embedded_fn);
+        ($res_fh, $res_fn, $embedded_fn, %res) = $self->_DecryptInlineBlock(
             %args,
             BlockHandle => $block_fh,
         );
@@ -1160,15 +1161,28 @@ sub _DecryptInlineBlock {
         Passphrase  => $args{'Passphrase'},
     );
 
+    my $embedded_fn = $self->_EmbeddedFilename( $res{'status'} );
+
     # if the decryption is fine but the signature is bad, then without this
     # status check we lose the decrypted text
     # XXX: add argument to the function to control this check
     delete $res{'message'} if $res{'status'} =~ /DECRYPTION_OKAY/;
 
-    return (undef, undef, %res) if $res{message};
+    return (undef, undef, $embedded_fn, %res) if $res{message};
 
     seek $tmp_fh, 0, 0;
-    return ($tmp_fh, $tmp_fn, %res);
+    return ($tmp_fh, $tmp_fn, $embedded_fn, %res);
+}
+
+# Encyrpted attachments have their filename obfuscated, and the original is
+# in the encrypted section. This returns the original filename returned in
+# the status-fd filehandle after decryption is done.
+sub _EmbeddedFilename {
+    my $self = shift;
+    my $status = shift;
+    my ($embedded_fn) = ( $status =~ /^\[GNUPG:\] PLAINTEXT \d+ \d+ (\S+)$/m );
+    $RT::Logger->debug("Embedded filename is $embedded_fn") if $embedded_fn;
+    return uri_unescape($embedded_fn);
 }
 
 sub DecryptAttachment {
@@ -1189,7 +1203,7 @@ sub DecryptAttachment {
     $args{'Data'}->bodyhandle->print( $tmp_fh );
     seek $tmp_fh, 0, 0;
 
-    my ($res_fh, $res_fn, %res) = $self->_DecryptInlineBlock(
+    my ($res_fh, $res_fn, $embedded_fn, %res) = $self->_DecryptInlineBlock(
         %args,
         BlockHandle => $tmp_fh,
     );
@@ -1200,13 +1214,30 @@ sub DecryptAttachment {
 
     my $head = $args{'Data'}->head;
 
-    # we can not trust original content type
-    # TODO: and don't have way to detect, so we just use octet-stream
-    # some clients may send .asc files (encryped) as text/plain
-    $head->mime_attr( "Content-Type" => 'application/octet-stream' );
+    if ($head->recommended_filename =~ /^PGPexch\.htm/i) {
+        # Special-case for handling old-style PGP plugin for Outlook style
+        # attachements of HTML content. For more, see:
+        # http://www.ietf.org/mail-archive/web/openpgp/current/msg01811.html
+        $head->mime_attr( "Content-Type" => 'text/html' );
+        $head->mime_attr( "Content-Disposition" => "inline" );
+    } elsif ($head->recommended_filename =~ /^PGPexch\.rtf/i) {
+        # Special-case for handling old-style PGP plugin for Outlook style
+        # attachements of RTF content. These remain as attachments and are
+        # not to be inlined; RTF is for Outlook only.
+        $head->mime_attr( "Content-Type" => 'text/rtf' );
+    } else {
+        # TODO
+        # The original Content-Type of the MIME part describes the PGP-encoded
+        # data, and not the file inside it. There is no way to know the type,
+        # so we just use octet-stream.
+        # Note, some clients may send .asc files (encryped) as text/plain.
+        # (MIME::Detect::Type could be useful here)
+        $head->mime_attr( "Content-Type" => 'application/octet-stream' );
+    }
 
-    my $filename = $head->recommended_filename;
-    $filename =~ s/\.${RE_FILE_EXTENSIONS}$//i;
+    my $filename = $embedded_fn || $head->recommended_filename;
+    my $file_extension_regex = join '|', @{ RT->Config->Get('GnuPG')->{FileExtensions} };
+    $filename =~ s/\.(?:$file_extension_regex)$//i;
     $head->mime_attr( $_ => $filename )
         foreach (qw(Content-Type.name Content-Disposition.filename));
 
@@ -1314,7 +1345,7 @@ my %parse_keyword = map { $_ => 1 } qw(
     DECRYPTION_FAILED DECRYPTION_OKAY
     BAD_PASSPHRASE GOOD_PASSPHRASE
     NO_SECKEY NO_PUBKEY
-    NO_RECP INV_RECP NODATA UNEXPECTED
+    NO_RECP INV_RECP NODATA UNEXPECTED FAILURE
 );
 
 # keywords we ignore without any messages as we parse them using other
@@ -1324,7 +1355,8 @@ my %ignore_keyword = map { $_ => 1 } qw(
     BEGIN_ENCRYPTION SIG_ID VALIDSIG
     ENC_TO BEGIN_DECRYPTION END_DECRYPTION GOODMDC
     TRUST_UNDEFINED TRUST_NEVER TRUST_MARGINAL TRUST_FULLY TRUST_ULTIMATE
-    DECRYPTION_INFO
+    DECRYPTION_INFO KEY_CONSIDERED DECRYPTION_KEY NEWSIG PINENTRY_LAUNCHED
+    IMPORT_OK DECRYPTION_COMPLIANCE_MODE PROGRESS INV_SGNR
 );
 
 sub ParseStatus {
@@ -1512,10 +1544,12 @@ sub ParseStatus {
                 Class          => $props[3],
                 Timestamp      => $props[4],
                 KeyFingerprint => $props[5],
-                User           => $user_hint{ $latest_user_main_key },
+                ( defined $latest_user_main_key ? ( User => $user_hint{$latest_user_main_key} ) : () )
             };
-            $res[-1]->{Message} .= ' by '. $user_hint{ $latest_user_main_key }->{'EmailAddress'}
-                if $user_hint{ $latest_user_main_key };
+            if ($latest_user_main_key) {
+                $res[-1]->{Message} .= ' by '. $user_hint{ $latest_user_main_key }->{'EmailAddress'}
+                    if $user_hint{ $latest_user_main_key };
+            }
         }
         elsif ( $keyword eq 'INV_RECP' ) {
             my ($rcode, $recipient) = split /\s+/, $args, 2;
@@ -1540,8 +1574,20 @@ sub ParseStatus {
                 Reason     => $reason,
             };
         }
+        elsif ( $keyword eq 'FAILURE' ) {
+            # FAILURE encrypt 167772218
+            my ($op, $rcode) = split /\s+/, $args;
+            my $reason = ReasonCodeToText( $keyword, $rcode );
+            push @res, {
+                Operation  => ucfirst($op),
+                Status     => 'ERROR',
+                Message    => "Failed to $op",
+                ReasonCode => $rcode,
+                Reason     => $reason,
+            };
+        }
         else {
-            $RT::Logger->warning("Keyword $keyword is unknown");
+            $RT::Logger->warning("Keyword $keyword is unknown : status line is $line");
             next;
         }
         $res[-1]{'Keyword'} = $keyword if @res && !$res[-1]{'Keyword'};
@@ -1709,7 +1755,7 @@ sub ParseKeysInfo {
             push @{ $res[-1]{'User'} ||= [] }, \%info;
         }
         elsif ( $tag eq 'fpr' ) {
-            $res[-1]{'Fingerprint'} = (split /:/, $line, 10)[8];
+            $res[-1]{'Fingerprint'} ||= (split /:/, $line, 10)[8];
         }
     }
     return @res;
