@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -70,6 +70,7 @@ our %FieldTypes = (
     Select => {
         sort_order => 10,
         selection_type => 1,
+        canonicalizes => 0,
 
         labels => [ 'Select multiple values',               # loc
                     'Select one value',                     # loc
@@ -83,8 +84,8 @@ our %FieldTypes = (
                 'Select box',              # loc
                 'List',                    # loc
             ],
-            single => [ 'Select box',              # loc
-                        'Dropdown',                # loc
+            single => [ 'Dropdown',                # loc
+                        'Select box',              # loc
                         'List',                    # loc
                       ]
         },
@@ -93,6 +94,7 @@ our %FieldTypes = (
     Freeform => {
         sort_order => 20,
         selection_type => 0,
+        canonicalizes => 1,
 
         labels => [ 'Enter multiple values',               # loc
                     'Enter one value',                     # loc
@@ -102,6 +104,7 @@ our %FieldTypes = (
     Text => {
         sort_order => 30,
         selection_type => 0,
+        canonicalizes => 1,
         labels         => [
                     'Fill in multiple text areas',                   # loc
                     'Fill in one text area',                         # loc
@@ -111,6 +114,7 @@ our %FieldTypes = (
     Wikitext => {
         sort_order => 40,
         selection_type => 0,
+        canonicalizes => 1,
         labels         => [
                     'Fill in multiple wikitext areas',                       # loc
                     'Fill in one wikitext area',                             # loc
@@ -121,6 +125,7 @@ our %FieldTypes = (
     Image => {
         sort_order => 50,
         selection_type => 0,
+        canonicalizes => 0,
         labels         => [
                     'Upload multiple images',               # loc
                     'Upload one image',                     # loc
@@ -130,6 +135,7 @@ our %FieldTypes = (
     Binary => {
         sort_order => 60,
         selection_type => 0,
+        canonicalizes => 0,
         labels         => [
                     'Upload multiple files',              # loc
                     'Upload one file',                    # loc
@@ -140,6 +146,7 @@ our %FieldTypes = (
     Combobox => {
         sort_order => 70,
         selection_type => 1,
+        canonicalizes => 1,
         labels         => [
                     'Combobox: Select or enter multiple values',               # loc
                     'Combobox: Select or enter one value',                     # loc
@@ -149,6 +156,7 @@ our %FieldTypes = (
     Autocomplete => {
         sort_order => 80,
         selection_type => 1,
+        canonicalizes => 1,
         labels         => [
                     'Enter multiple values with autocompletion',               # loc
                     'Enter one value with autocompletion',                     # loc
@@ -159,6 +167,7 @@ our %FieldTypes = (
     Date => {
         sort_order => 90,
         selection_type => 0,
+        canonicalizes => 0,
         labels         => [
                     'Select multiple dates',              # loc
                     'Select date',                        # loc
@@ -168,6 +177,7 @@ our %FieldTypes = (
     DateTime => {
         sort_order => 100,
         selection_type => 0,
+        canonicalizes => 0,
         labels         => [
                     'Select multiple datetimes',                  # loc
                     'Select datetime',                            # loc
@@ -178,6 +188,7 @@ our %FieldTypes = (
     IPAddress => {
         sort_order => 110,
         selection_type => 0,
+        canonicalizes => 0,
 
         labels => [ 'Enter multiple IP addresses',                    # loc
                     'Enter one IP address',                           # loc
@@ -187,6 +198,7 @@ our %FieldTypes = (
     IPAddressRange => {
         sort_order => 120,
         selection_type => 0,
+        canonicalizes => 0,
 
         labels => [ 'Enter multiple IP address ranges',                          # loc
                     'Enter one IP address range',                                # loc
@@ -208,12 +220,14 @@ __PACKAGE__->RegisterLookupType( 'RT::Group' => "Groups", );
 __PACKAGE__->RegisterBuiltInGroupings(
     'RT::Ticket'    => [ qw(Basics Dates Links People) ],
     'RT::User'      => [ 'Identity', 'Access control', 'Location', 'Phones' ],
+    'RT::Group'     => [ 'Basics' ],
 );
 
 __PACKAGE__->AddRight( General => SeeCustomField         => 'View custom fields'); # loc
 __PACKAGE__->AddRight( Admin   => AdminCustomField       => 'Create, modify and delete custom fields'); # loc
 __PACKAGE__->AddRight( Admin   => AdminCustomFieldValues => 'Create, modify and delete custom fields values'); # loc
 __PACKAGE__->AddRight( Staff   => ModifyCustomField      => 'Add, modify and delete custom field values for objects'); # loc
+__PACKAGE__->AddRight( Staff   => SetInitialCustomField  => 'Add custom field values only at object creation time'); # loc
 
 =head1 NAME
 
@@ -234,6 +248,7 @@ Create takes a hash of values and creates a row in the database:
   varchar(255) 'Description'.
   int(11) 'SortOrder'.
   varchar(255) 'LookupType'.
+  varchar(255) 'EntryHint'.
   smallint(6) 'Disabled'.
 
 C<LookupType> is generally the result of either
@@ -244,15 +259,19 @@ C<RT::Ticket->CustomFieldLookupType> or C<RT::Transaction->CustomFieldLookupType
 sub Create {
     my $self = shift;
     my %args = (
-        Name        => '',
-        Type        => '',
-        MaxValues   => 0,
-        Pattern     => '',
-        Description => '',
-        Disabled    => 0,
-        LookupType  => '',
-        LinkValueTo => '',
+        Name                   => '',
+        Type                   => '',
+        MaxValues              => 0,
+        Pattern                => '',
+        Description            => '',
+        Disabled               => 0,
+        SortOrder              => 0,
+        LookupType             => '',
+        LinkValueTo            => '',
         IncludeContentForValue => '',
+        EntryHint              => undef,
+        UniqueValues           => 0,
+        CanonicalizeClass      => undef,
         @_,
     );
 
@@ -322,19 +341,31 @@ sub Create {
         }
     }
 
+    if ( $args{'CanonicalizeClass'} ||= undef ) {
+        return (0, $self->loc("This custom field can not have a canonicalizer"))
+            unless $self->IsCanonicalizeType( $args{'Type'} );
+
+        unless ( $self->ValidateCanonicalizeClass( $args{'CanonicalizeClass'} ) ) {
+            return (0, $self->loc("Invalid custom field values canonicalizer"));
+        }
+    }
+
     $args{'Disabled'} ||= 0;
 
     (my $rv, $msg) = $self->SUPER::Create(
-        Name        => $args{'Name'},
-        Type        => $args{'Type'},
-        RenderType  => $args{'RenderType'},
-        MaxValues   => $args{'MaxValues'},
-        Pattern     => $args{'Pattern'},
-        BasedOn     => $args{'BasedOn'},
-        ValuesClass => $args{'ValuesClass'},
-        Description => $args{'Description'},
-        Disabled    => $args{'Disabled'},
-        LookupType  => $args{'LookupType'},
+        Name              => $args{'Name'},
+        Type              => $args{'Type'},
+        RenderType        => $args{'RenderType'},
+        MaxValues         => $args{'MaxValues'},
+        Pattern           => $args{'Pattern'},
+        BasedOn           => $args{'BasedOn'},
+        ValuesClass       => $args{'ValuesClass'},
+        Description       => $args{'Description'},
+        Disabled          => $args{'Disabled'},
+        SortOrder         => $args{'SortOrder'},
+        LookupType        => $args{'LookupType'},
+        UniqueValues      => $args{'UniqueValues'},
+        CanonicalizeClass => $args{'CanonicalizeClass'},
     );
 
     if ($rv) {
@@ -342,6 +373,8 @@ sub Create {
             $self->SetLinkValueTo($args{'LinkValueTo'});
         }
 
+        $self->SetEntryHint( $args{EntryHint} // $self->FriendlyType );
+
         if ( exists $args{'IncludeContentForValue'}) {
             $self->SetIncludeContentForValue($args{'IncludeContentForValue'});
         }
@@ -606,7 +639,7 @@ sub Values {
     my $cf_values = $class->new( $self->CurrentUser );
     $cf_values->SetCustomFieldObject( $self );
     # if the user has no rights, return an empty object
-    if ( $self->id && $self->CurrentUserHasRight( 'SeeCustomField') ) {
+    if ( $self->id && $self->CurrentUserCanSee ) {
         $cf_values->LimitToCustomField( $self->Id );
     } else {
         $cf_values->Limit( FIELD => 'id', VALUE => 0, SUBCLAUSE => 'acl' );
@@ -665,11 +698,11 @@ sub DeleteValue {
         return (0, $self->loc("That is not a value for this custom field"));
     }
 
-    my $retval = $val_to_del->Delete;
-    unless ( $retval ) {
+    my ($ok, $msg) = $val_to_del->Delete;
+    unless ( $ok ) {
         return (0, $self->loc("Custom field value could not be deleted"));
     }
-    return ($retval, $self->loc("Custom field value deleted"));
+    return ($ok, $self->loc("Custom field value deleted"));
 }
 
 
@@ -723,18 +756,31 @@ sub Types {
 
 =head2 IsSelectionType 
 
-Retuns a boolean value indicating whether the C<Values> method makes sense
+Returns a boolean value indicating whether the C<Values> method makes sense
 to this Custom Field.
 
 =cut
 
 sub IsSelectionType {
     my $self = shift;
-    my $type = @_? shift : $self->Type;
+    my $type = @_ ? shift : $self->Type;
     return undef unless $type;
     return $FieldTypes{$type}->{selection_type};
 }
 
+=head2 IsCanonicalizeType
+
+Returns a boolean value indicating whether the type of this custom field
+permits using a canonicalizer.
+
+=cut
+
+sub IsCanonicalizeType {
+    my $self = shift;
+    my $type = @_ ? shift : $self->Type;
+    return undef unless $type;
+    return $FieldTypes{$type}->{canonicalizes};
+}
 
 
 =head2 IsExternalValues
@@ -752,6 +798,14 @@ sub ValuesClass {
     return $self->_Value( ValuesClass => @_ ) || 'RT::CustomFieldValues';
 }
 
+=head2 SetValuesClass CLASS
+
+Writer method for the ValuesClass field; validates that the custom field can
+use a ValuesClass, and that the provided ValuesClass passes
+L</ValidateValuesClass>.
+
+=cut
+
 sub SetValuesClass {
     my $self = shift;
     my $class = shift || 'RT::CustomFieldValues';
@@ -769,6 +823,17 @@ sub SetValuesClass {
     return $self->_Set( Field => 'ValuesClass', Value => $class, @_ );
 }
 
+=head2 ValidateValuesClass CLASS
+
+Validates a potential ValuesClass value; the ValuesClass may be C<undef> or
+the string C<"RT::CustomFieldValues"> (both of which make this custom field
+use the ordinary values implementation), or a class name in the listed in
+the L<RT_Config/@CustomFieldValuesSources> setting.
+
+Returns true if valid; false if invalid.
+
+=cut
+
 sub ValidateValuesClass {
     my $self = shift;
     my $class = shift;
@@ -778,6 +843,50 @@ sub ValidateValuesClass {
     return undef;
 }
 
+=head2 SetCanonicalizeClass CLASS
+
+Writer method for the CanonicalizeClass field; validates that the custom
+field can use a CanonicalizeClass, and that the provided CanonicalizeClass
+passes L</ValidateCanonicalizeClass>.
+
+=cut
+
+sub SetCanonicalizeClass {
+    my $self = shift;
+    my $class = shift;
+
+    if ( !$class ) {
+        return $self->_Set( Field => 'CanonicalizeClass', Value => undef, @_ );
+    }
+
+    return (0, $self->loc("This custom field can not have a canonicalizer"))
+        unless $self->IsCanonicalizeType;
+
+    unless ( $self->ValidateCanonicalizeClass( $class ) ) {
+        return (0, $self->loc("Invalid custom field values canonicalizer"));
+    }
+    return $self->_Set( Field => 'CanonicalizeClass', Value => $class, @_ );
+}
+
+=head2 ValidateCanonicalizeClass CLASS
+
+Validates a potential CanonicalizeClass value; the CanonicalizeClass may be
+C<undef> (which make this custom field use no special canonicalization), or
+a class name in the listed in the
+L<RT_Config/@CustomFieldValuesCanonicalizers> setting.
+
+Returns true if valid; false if invalid.
+
+=cut
+
+sub ValidateCanonicalizeClass {
+    my $self = shift;
+    my $class = shift;
+
+    return 1 if !$class;
+    return 1 if grep $class eq $_, RT->Config->Get('CustomFieldValuesCanonicalizers');
+    return undef;
+}
 
 =head2 FriendlyType [TYPE, MAX_VALUES]
 
@@ -820,14 +929,6 @@ sub ValidateType {
     my $self = shift;
     my $type = shift;
 
-    if ( $type =~ s/(?:Single|Multiple)$// ) {
-        RT->Deprecated(
-            Arguments => "suffix 'Single' or 'Multiple'",
-            Instead   => "MaxValues",
-            Remove    => "4.4",
-        );
-    }
-
     if ( $FieldTypes{$type} ) {
         return 1;
     }
@@ -840,15 +941,11 @@ sub ValidateType {
 sub SetType {
     my $self = shift;
     my $type = shift;
-    if ($type =~ s/(?:(Single)|Multiple)$//) {
-        RT->Deprecated(
-            Arguments => "suffix 'Single' or 'Multiple'",
-            Instead   => "MaxValues",
-            Remove    => "4.4",
-        );
-        $self->SetMaxValues($1 ? 1 : 0);
-    }
-    $self->_Set(Field => 'Type', Value =>$type);
+    my $need_to_update_hint;
+    $need_to_update_hint = 1 if $self->EntryHint && $self->EntryHint eq $self->FriendlyType;
+    my ( $ret, $msg ) = $self->_Set( Field => 'Type', Value => $type );
+    $self->SetEntryHint($self->FriendlyType) if $need_to_update_hint && $ret;
+    return ( $ret, $msg );
 }
 
 =head2 SetPattern STRING
@@ -1032,12 +1129,15 @@ sub ValidateContextObject {
 
 sub _Set {
     my $self = shift;
-
+    my %args = @_;
     unless ( $self->CurrentUserHasRight('AdminCustomField') ) {
         return ( 0, $self->loc('Permission Denied') );
     }
-    return $self->SUPER::_Set( @_ );
-
+    my ($ret, $msg) = $self->SUPER::_Set( @_ );
+    if ( $args{Field} =~ /^(?:MaxValues|Type|LookupType|ValuesClass|CanonicalizeClass)$/ ) {
+        $self->CleanupDefaultValues;
+    }
+    return ($ret, $msg);
 }
 
 
@@ -1054,7 +1154,7 @@ sub _Value {
     return undef unless $self->id;
 
     # we need to do the rights check
-    unless ( $self->CurrentUserHasRight('SeeCustomField') ) {
+    unless ( $self->CurrentUserCanSee ) {
         $RT::Logger->debug(
             "Permission denied. User #". $self->CurrentUser->id
             ." has no SeeCustomField right on CF #". $self->id
@@ -1073,6 +1173,22 @@ Takes a boolean.
 
 =cut
 
+sub SetDisabled {
+    my $self = shift;
+    my $val = shift;
+
+    my ($status, $msg) = $self->_Set(Field => 'Disabled', Value => $val);
+
+    unless ($status) {
+        return ($status, $msg);
+    }
+
+    if ( $val == 1 ) {
+        return (1, $self->loc("Disabled"));
+    } else {
+        return (1, $self->loc("Enabled"));
+    }
+}
 
 =head2 SetTypeComposite
 
@@ -1329,8 +1445,8 @@ sub ObjectTypeFromLookupType {
 
 sub CollectionClassFromLookupType {
     my $self = shift;
+    my $record_class = shift || $self->RecordClassFromLookupType;
 
-    my $record_class = $self->RecordClassFromLookupType;
     return undef unless $record_class;
 
     my $collection_class;
@@ -1459,13 +1575,6 @@ sub IsOnlyGlobal {
     return ($self->LookupType =~ /^RT::(?:Group|User)/io);
 
 }
-sub ApplyGlobally {
-    RT->Deprecated(
-        Instead   => "IsOnlyGlobal",
-        Remove    => "4.4",
-    );
-    return shift->IsOnlyGlobal(@_);
-}
 
 =head1 AddedTo
 
@@ -1482,13 +1591,6 @@ sub AddedTo {
     return RT::ObjectCustomField->new( $self->CurrentUser )
         ->AddedTo( CustomField => $self );
 }
-sub AppliedTo {
-    RT->Deprecated(
-        Instead   => "AddedTo",
-        Remove    => "4.4",
-    );
-    shift->AddedTo(@_);
-};
 
 =head1 NotAddedTo
 
@@ -1505,13 +1607,6 @@ sub NotAddedTo {
     return RT::ObjectCustomField->new( $self->CurrentUser )
         ->NotAddedTo( CustomField => $self );
 }
-sub NotAppliedTo {
-    RT->Deprecated(
-        Instead   => "NotAddedTo",
-        Remove    => "4.4",
-    );
-    shift->NotAddedTo(@_)
-};
 
 =head2 IsAdded
 
@@ -1529,13 +1624,6 @@ sub IsAdded {
     return undef unless $ocf->id;
     return $ocf;
 }
-sub IsApplied {
-    RT->Deprecated(
-        Instead   => "IsAdded",
-        Remove    => "4.4",
-    );
-    shift->IsAdded(@_);
-};
 
 sub IsGlobal { return shift->IsAdded(0) }
 
@@ -1575,9 +1663,18 @@ sub AddToObject {
     }
 
     my $ocf = RT::ObjectCustomField->new( $self->CurrentUser );
-    my ( $oid, $msg ) = $ocf->Add(
+    my $oid = $ocf->Add(
         CustomField => $self->id, ObjectId => $id,
     );
+
+    my $msg;
+    # If object has no id, it represents all objects
+    if ($object->id) {
+        $msg = $self->loc( 'Added custom field [_1] to [_2].', $self->Name, $object->Name );
+    } else {
+        $msg = $self->loc( 'Globally added custom field [_1].', $self->Name );
+    }
+
     return ( $oid, $msg );
 }
 
@@ -1608,9 +1705,15 @@ sub RemoveFromObject {
         return ( 0, $self->loc("This custom field cannot be added to that object") );
     }
 
-    # XXX: Delete doesn't return anything
-    my ( $oid, $msg ) = $ocf->Delete;
-    return ( $oid, $msg );
+    my ($ok, $msg) = $ocf->Delete;
+    return ($ok, $msg) unless $ok;
+
+    # If object has no id, it represents all objects
+    if ($object->id) {
+        return (1, $self->loc( 'Removed custom field [_1] from [_2].', $self->Name, $object->Name ) );
+    } else {
+        return (1, $self->loc( 'Globally removed custom field [_1].', $self->Name ) );
+    }
 }
 
 
@@ -1638,11 +1741,15 @@ sub AddValueForObject {
         Content      => undef,
         LargeContent => undef,
         ContentType  => undef,
+        ForCreation  => 0,
         @_
     );
     my $obj = $args{'Object'} or return ( 0, $self->loc('Invalid object') );
 
-    unless ( $self->CurrentUserHasRight('ModifyCustomField') ) {
+    unless (
+        $self->CurrentUserHasRight('ModifyCustomField') ||
+        ($args{ForCreation} && $self->CurrentUserHasRight('SetInitialCustomField'))
+    ) {
         return ( 0, $self->loc('Permission Denied') );
     }
 
@@ -1654,24 +1761,41 @@ sub AddValueForObject {
 
     if ( $self->MaxValues ) {
         my $current_values = $self->ValuesForObject($obj);
-        my $extra_values = ( $current_values->Count + 1 ) - $self->MaxValues;
 
         # (The +1 is for the new value we're adding)
+        my $extra_values = ( $current_values->Count + 1 ) - $self->MaxValues;
 
-        # If we have a set of current values and we've gone over the maximum
-        # allowed number of values, we'll need to delete some to make room.
-        # which former values are blown away is not guaranteed
 
-        while ($extra_values) {
-            my $extra_item = $current_values->Next;
-            unless ( $extra_item->id ) {
-                $RT::Logger->crit( "We were just asked to delete "
-                    ."a custom field value that doesn't exist!" );
-                $RT::Handle->Rollback();
-                return (undef);
+        # Could have a negative value if MaxValues is greater than count
+        if ( $extra_values > 0 ) {
+
+            # If we have a set of current values and we've gone over the maximum
+            # allowed number of values, we'll need to delete some to make room.
+            # which former values are blown away is not guaranteed
+
+            while ($extra_values) {
+                my $extra_item = $current_values->Next;
+                unless ( $extra_item && $extra_item->id ) {
+                    $RT::Logger->crit( "We were just asked to delete "
+                        ."a custom field value that doesn't exist!" );
+                    $RT::Handle->Rollback();
+                    return (undef);
+                }
+                $extra_item->Delete;
+                $extra_values--;
             }
-            $extra_item->Delete;
-            $extra_values--;
+        }
+    }
+
+    if ($self->UniqueValues) {
+        my $class = $self->CollectionClassFromLookupType($self->ObjectTypeFromLookupType);
+        my $collection = $class->new(RT->SystemUser);
+        $collection->LimitCustomField(CUSTOMFIELD => $self->Id, OPERATOR => '=', VALUE => $args{'LargeContent'} // $args{'Content'});
+
+        if ($collection->Count) {
+            $RT::Logger->debug( "Non-unique custom field value for CF #" . $self->Id ." with object custom field value " . $collection->First->Id );
+            $RT::Handle->Rollback();
+            return ( 0, $self->loc('That is not a unique value') );
         }
     }
 
@@ -1700,14 +1824,33 @@ sub _CanonicalizeValue {
     my $self = shift;
     my $args = shift;
 
-    my $type = $self->_Value('Type');
+    my $type = $self->__Value('Type');
     return 1 unless $type;
 
+    $self->_CanonicalizeValueWithCanonicalizer($args);
+
     my $method = '_CanonicalizeValue'. $type;
     return 1 unless $self->can($method);
     $self->$method($args);
 }
 
+sub _CanonicalizeValueWithCanonicalizer {
+    my $self = shift;
+    my $args = shift;
+
+    my $class = $self->__Value('CanonicalizeClass') or return 1;
+
+    $class->require or die "Can't load $class: $@";
+    my $canonicalizer = $class->new($self->CurrentUser);
+
+    $args->{'Content'} = $canonicalizer->CanonicalizeValue(
+        CustomField => $self,
+        Content     => $args->{'Content'},
+    );
+
+    return 1;
+}
+
 sub _CanonicalizeValueDateTime {
     my $self    = shift;
     my $args    = shift;
@@ -1845,9 +1988,9 @@ sub DeleteValueForObject {
 
     # delete it
 
-    my $ret = $oldval->Delete();
-    unless ($ret) {
-        return(0, $self->loc("Custom field value could not be found"));
+    my ($ok, $msg) = $oldval->Delete();
+    unless ($ok) {
+        return(0, $self->loc("Custom field value could not be deleted"));
     }
     return($oldval->Id, $self->loc("Custom field value deleted"));
 }
@@ -1864,7 +2007,7 @@ sub ValuesForObject {
     my $object = shift;
 
     my $values = RT::ObjectCustomFieldValues->new($self->CurrentUser);
-    unless ($self->id and $self->CurrentUserHasRight('SeeCustomField')) {
+    unless ($self->id and $self->CurrentUserCanSee) {
         # Return an empty object if they have no rights to see
         $values->Limit( FIELD => "id", VALUE => 0, SUBCLAUSE => "ACL" );
         return ($values);
@@ -1876,6 +2019,26 @@ sub ValuesForObject {
     return ($values);
 }
 
+=head2 CurrentUserCanSee
+
+If the user has SeeCustomField they can see this custom field and its details.
+
+Otherwise, if the user has SetInitialCustomField and this is being used in a
+"create" context, then they can see this custom field and its details. This
+allows you to set up custom fields that are only visible on create pages and
+are then inaccessible.
+
+=cut
+
+sub CurrentUserCanSee {
+    my $self = shift;
+    return 1 if $self->CurrentUserHasRight('SeeCustomField');
+
+    return 1 if $self->{include_set_initial}
+             && $self->CurrentUserHasRight('SetInitialCustomField');
+
+    return 0;
+}
 
 =head2 RegisterLookupType LOOKUPTYPE FRIENDLYNAME
 
@@ -1902,16 +2065,6 @@ sub RegisterLookupType {
     $FRIENDLY_LOOKUP_TYPES{$path} = $friendly_name;
 }
 
-sub _ForObjectType {
-    RT->Deprecated(
-        Instead => 'RegisterLookupType',
-        Remove  => '4.4',
-    );
-    my $self = shift;
-    $self->RegisterLookupType(@_);
-}
-
-
 =head2 IncludeContentForValue [VALUE] (and SetIncludeContentForValue)
 
 Gets or sets the  C<IncludeContentForValue> for this custom field. RT
@@ -1975,7 +2128,7 @@ sub _URLTemplate {
         }
         return ( 1, $self->loc('Updated') );
     } else {
-        unless ( $self->id && $self->CurrentUserHasRight('SeeCustomField') ) {
+        unless ( $self->id && $self->CurrentUserCanSee ) {
             return (undef);
         }
 
@@ -1997,7 +2150,7 @@ sub SetBasedOn {
     $cf->Load( ref $value ? $value->id : $value );
 
     return (0, "Permission Denied")
-        unless $cf->id && $cf->CurrentUserHasRight('SeeCustomField');
+        unless $cf->id && $cf->CurrentUserCanSee;
 
     # XXX: Remove this restriction once we support lists and cascaded selects
     if ( $self->RenderType =~ /List/ ) {
@@ -2019,9 +2172,179 @@ sub BasedOnObj {
 }
 
 
+sub SupportDefaultValues {
+    my $self = shift;
+    return 0 unless $self->id;
+    return 0 unless $self->LookupType =~ /RT::(?:Ticket|Transaction|Asset)$/;
+    return $self->Type !~ /^(?:Image|Binary)$/;
+}
 
+sub DefaultValues {
+    my $self = shift;
+    my %args = (
+        Object => RT->System,
+        @_,
+    );
+    my $attr = $args{Object}->FirstAttribute('CustomFieldDefaultValues');
+    my $values;
+    $values = $attr->Content->{$self->id} if $attr && $attr->Content;
+    return $values if defined $values;
 
+    if ( !$args{Object}->isa( 'RT::System' ) ) {
+        my $system_attr = RT::System->FirstAttribute( 'CustomFieldDefaultValues' );
+        $values = $system_attr->Content->{$self->id} if $system_attr && $system_attr->Content;
+        return $values if defined $values;
+    }
+    return undef;
+}
 
+sub SetDefaultValues {
+    my $self = shift;
+    my %args = (
+        Object => RT->System,
+        Values => undef,
+        @_,
+    );
+    my $attr = $args{Object}->FirstAttribute( 'CustomFieldDefaultValues' );
+    my ( $old_values, $old_content, $new_values );
+    if ( $attr && $attr->Content ) {
+        $old_content = $attr->Content;
+        $old_values = $old_content->{ $self->id };
+    }
+
+    if ( !$args{Object}->isa( 'RT::System' ) && !defined $old_values ) {
+        my $system_attr = RT::System->FirstAttribute( 'CustomFieldDefaultValues' );
+        if ( $system_attr && $system_attr->Content ) {
+            $old_values = $system_attr->Content->{ $self->id };
+        }
+    }
+
+    if ( defined $old_values && length $old_values ) {
+        $old_values = join ', ', @$old_values if ref $old_values eq 'ARRAY';
+    }
+
+    $new_values = $args{Values};
+    if ( defined $new_values && length $new_values ) {
+        $new_values = join ', ', @$new_values if ref $new_values eq 'ARRAY';
+    }
+
+    return 1 if ( $new_values // '' ) eq ( $old_values // '' );
+
+    my ($ret, $msg) = $args{Object}->SetAttribute(
+        Name    => 'CustomFieldDefaultValues',
+        Content => {
+            %{ $old_content || {} }, $self->id => $args{Values},
+        },
+    );
+
+    $old_values = $self->loc('(no value)') unless defined $old_values && length $old_values;
+    $new_values = $self->loc( '(no value)' ) unless defined $new_values && length $new_values;
+
+    if ( $ret ) {
+        return ( $ret, $self->loc( 'Default values changed from [_1] to [_2]', $old_values, $new_values ) );
+    }
+    else {
+        return ( $ret, $self->loc( "Can't change default values from [_1] to [_2]: [_3]", $old_values, $new_values, $msg ) );
+    }
+}
+
+sub CleanupDefaultValues {
+    my $self  = shift;
+    my $attrs = RT::Attributes->new( $self->CurrentUser );
+    $attrs->Limit( FIELD => 'Name', VALUE => 'CustomFieldDefaultValues' );
+
+    my @values;
+    if ( $self->Type eq 'Select' ) {
+        # Select has a limited list valid values, we need to exclude invalid ones
+        @values = map { $_->Name } @{ $self->Values->ItemsArrayRef || [] };
+    }
+
+    while ( my $attr = $attrs->Next ) {
+        my $content = $attr->Content;
+        next unless $content;
+        my $changed;
+        if ( $self->SupportDefaultValues ) {
+            if ( $self->MaxValues == 1 && ref $content->{ $self->id } eq 'ARRAY' ) {
+                $content->{ $self->id } = $content->{ $self->id }[ 0 ];
+                $changed = 1;
+            }
+
+            my $default_values = $content->{ $self->id };
+            if ( $default_values ) {
+                if ( $self->Type eq 'Select' ) {
+                    if ( ref $default_values ne 'ARRAY' && $default_values =~ /\n/ ) {
+
+                        # e.g. multiple values Freeform cf has 2 default values: foo and "bar",
+                        # the values will be stored as "foo\nbar".  so we need to convert it to ARRAY for Select cf.
+                        # this could happen when we change a Freeform cf into a Select one
+
+                        $default_values = [ split /\s*\n+\s*/, $default_values ];
+                        $content->{ $self->id } = $default_values;
+                        $changed = 1;
+                    }
+
+                    if ( ref $default_values eq 'ARRAY' ) {
+                        my @new_defaults;
+                        for my $default ( @$default_values ) {
+                            if ( grep { $_ eq $default } @values ) {
+                                push @new_defaults, $default;
+                            }
+                            else {
+                                $changed = 1;
+                            }
+                        }
+
+                        $content->{ $self->id } = \@new_defaults if $changed;
+                    }
+                    elsif ( !grep { $_ eq $default_values } @values ) {
+                        delete $content->{ $self->id };
+                        $changed = 1;
+                    }
+                }
+                else {
+                    # ARRAY default values only happen for Select cf. we need to convert it to a scalar for other cfs.
+                    # this could happen when we change a Select cf into a Freeform one
+
+                    if ( ref $default_values eq 'ARRAY' ) {
+                        $content->{ $self->id } = join "\n", @$default_values;
+                        $changed = 1;
+                    }
+
+                    if ($self->MaxValues == 1) {
+                        my $args = { Content => $default_values };
+                        $self->_CanonicalizeValueWithCanonicalizer($args);
+                        if ($args->{Content} ne $default_values) {
+                            $content->{ $self->id } = $default_values;
+                            $changed = 1;
+                        }
+                    }
+                    else {
+                        my @new_values;
+                        my $multi_changed = 0;
+                        for my $value (split /\s*\n+\s*/, $default_values) {
+                            my $args = { Content => $value };
+                            $self->_CanonicalizeValueWithCanonicalizer($args);
+                            push @new_values, $args->{Content};
+                            $multi_changed = 1 if $args->{Content} ne $value;
+                        }
+
+                        if ($multi_changed) {
+                            $content->{ $self->id } = join "\n", @new_values;
+                            $changed = 1;
+                        }
+                    }
+                }
+            }
+        }
+        else {
+            if ( exists $content->{ $self->id } ) {
+                delete $content->{ $self->id };
+                $changed = 1;
+            }
+        }
+        $attr->SetContent( $content ) if $changed;
+    }
+}
 
 =head2 id
 
@@ -2191,6 +2514,16 @@ Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
 (In the database, LookupType will be stored as a varchar(255).)
 
 
+=cut
+
+=head2 SetEntryHint VALUE
+
+
+Set EntryHint to VALUE.
+Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
+(In the database, EntryHint will be stored as a varchar(255).)
+
+
 =cut
 
 
@@ -2274,6 +2607,12 @@ sub _CoreAccessible {
         {read => 1, write => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => '0'},
         LookupType => 
         {read => 1, write => 1, sql_type => 12, length => 255,  is_blob => 0,  is_numeric => 0,  type => 'varchar(255)', default => ''},
+        EntryHint =>
+        {read => 1, write => 1, sql_type => 12, length => 255,  is_blob => 0, is_numeric => 0,  type => 'varchar(255)', default => undef },
+        UniqueValues =>
+        {read => 1, write => 1, sql_type => 5, length => 6,  is_blob => 0,  is_numeric => 1,  type => 'smallint(6)', default => '0'},
+        CanonicalizeClass =>
+        {read => 1, write => 1, sql_type => 12, length => 64,  is_blob => 0,  is_numeric => 0,  type => 'varchar(64)', default => ''},
         Creator => 
         {read => 1, auto => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => '0'},
         Created => 
@@ -2317,6 +2656,11 @@ sub __DependsOn {
 # Custom field values
     push( @$list, $self->Values );
 
+# Applications of this CF
+    my $applied = RT::ObjectCustomFields->new( $self->CurrentUser );
+    $applied->LimitToCustomField( $self->Id );
+    push @$list, $applied;
+
 # Ticket custom field values
     my $objs = RT::ObjectCustomFieldValues->new( $self->CurrentUser );
     $objs->LimitToCustomField( $self->Id );
@@ -2331,6 +2675,91 @@ sub __DependsOn {
     return $self->SUPER::__DependsOn( %args );
 }
 
+=head2 LoadByNameAndCatalog
+
+Loads the described asset custom field, if one is found, into the current
+object.  This method only consults custom fields applied to L<RT::Catalog> for
+L<RT::Asset> objects.
+
+Takes a hash with the keys:
+
+=over
+
+=item Name
+
+A L<RT::CustomField> ID or Name which applies to L<assets|RT::Asset>.
+
+=item Catalog
+
+Optional.  An L<RT::Catalog> ID or Name.
+
+=back
+
+If Catalog is specified, only a custom field added to that Catalog will be loaded.
+
+If Catalog is C<0>, only global asset custom fields will be loaded.
+
+If no Catalog is specified, all asset custom fields are searched including
+global and catalog-specific CFs.
+
+Please note that this method may load a Disabled custom field if no others
+matching the same criteria are found.  Enabled CFs are preferentially loaded.
+
+=cut
+
+# To someday be merged into RT::CustomField::LoadByName
+sub LoadByNameAndCatalog {
+    my $self = shift;
+    my %args = (
+                Catalog => undef,
+                Name  => undef,
+                @_,
+               );
+
+    unless ( defined $args{'Name'} && length $args{'Name'} ) {
+        $RT::Logger->error("Couldn't load Custom Field without Name");
+        return wantarray ? (0, $self->loc("No name provided")) : 0;
+    }
+
+    # if we're looking for a catalog by name, make it a number
+    if ( defined $args{'Catalog'} && ($args{'Catalog'} =~ /\D/ || !$self->ContextObject) ) {
+        my $CatalogObj = RT::Catalog->new( $self->CurrentUser );
+        my ($ok, $msg) = $CatalogObj->Load( $args{'Catalog'} );
+        if ( $ok ){
+            $args{'Catalog'} = $CatalogObj->Id;
+        }
+        elsif ($args{'Catalog'}) {
+            RT::Logger->error("Unable to load catalog " . $args{'Catalog'} . $msg);
+            return (0, $msg);
+        }
+        $self->SetContextObject( $CatalogObj )
+          unless $self->ContextObject;
+    }
+
+    my $CFs = RT::CustomFields->new( $self->CurrentUser );
+    $CFs->SetContextObject( $self->ContextObject );
+    my $field = $args{'Name'} =~ /\D/? 'Name' : 'id';
+    $CFs->Limit( FIELD => $field, VALUE => $args{'Name'}, CASESENSITIVE => 0);
+
+    # Limit to catalog, if provided. This will also limit to RT::Asset types.
+    $CFs->LimitToCatalog( $args{'Catalog'} );
+
+    # When loading by name, we _can_ load disabled fields, but prefer
+    # non-disabled fields.
+    $CFs->FindAllRows;
+    $CFs->OrderByCols(
+                      {
+                       FIELD => "Disabled", ORDER => 'ASC' },
+                     );
+
+    # We only want one entry.
+    $CFs->RowsPerPage(1);
+
+    return (0, $self->loc("Not found")) unless my $first = $CFs->First;
+    return $self->LoadById( $first->id );
+}
+
+
 RT::Base->_ImportOverlays();
 
 1;
