@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2018 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -246,6 +246,8 @@ sub child {
             $path =~ s/index\.html$//;
             $path =~ s/\/+$//;
 
+            require URI::Escape;
+            $base_path = URI::Escape::uri_unescape($base_path);
             if ( $path eq $base_path ) {
                 $self->{children}{$key}->active(1);
             }
