@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -99,6 +99,10 @@ will appear as a key whose value is 1.
 
 sub GetReferencedQueues {
     my $self = shift;
+    my %args = (
+        CurrentUser => '',
+        @_
+    );
 
     my $queues = {};
 
@@ -110,18 +114,66 @@ sub GetReferencedQueues {
             return unless $node->isLeaf;
 
             my $clause = $node->getNodeValue();
-            return unless $clause->{Key} eq 'Queue';
-            return unless $clause->{Op} eq '=';
-
-            my $value = $clause->{Value};
-            $value =~ s/\\(.)/$1/g if $value =~ s/^'(.*)'$/$1/;
-            $queues->{ $value } = 1;
+            if ( $clause->{Key} =~ /^(?:Ticket)?Queue$/ ) {
+                if ( $clause->{Op} eq '=' ) {
+                    $queues->{ $clause->{Value} } ||= 1;
+                }
+                elsif ( $clause->{Op} =~ /^LIKE$/i ) {
+                    my $qs = RT::Queues->new( $args{CurrentUser} || $HTML::Mason::Commands::session{CurrentUser} );
+                    $qs->Limit( FIELD => 'Name', VALUE => $clause->{Value}, OPERATOR => 'LIKE', CASESENSITIVE => 0 );
+                    while ( my $q = $qs->Next ) {
+                        next unless $q->id;
+                        $queues->{ $q->id } ||= 1;
+                    }
+                }
+            }
+            elsif ( $clause->{Key} eq 'Lifecycle' ) {
+                if ( $clause->{Op} eq '=' ) {
+                    my $qs = RT::Queues->new( $args{CurrentUser} || $HTML::Mason::Commands::session{CurrentUser} );
+                    $qs->Limit( FIELD => 'Lifecycle', VALUE => $clause->{Value} );
+                    while ( my $q = $qs->Next ) {
+                        next unless $q->id;
+                        $queues->{ $q->id } ||= 1;
+                    }
+                }
+            }
+            return;
         }
     );
 
     return $queues;
 }
 
+=head2 GetReferencedCatalogs
+
+Returns a hash reference; each catalog referenced with an '=' operation
+will appear as a key whose value is 1.
+
+=cut
+
+sub GetReferencedCatalogs {
+    my $self = shift;
+
+    my $catalogs = {};
+
+    $self->traverse(
+        sub {
+            my $node = shift;
+
+            return if $node->isRoot;
+            return unless $node->isLeaf;
+
+            my $clause = $node->getNodeValue();
+            return unless $clause->{ Key } eq 'Catalog';
+            return unless $clause->{ Op } eq '=';
+
+            $catalogs->{ $clause->{ Value } } = 1;
+        }
+    );
+
+    return $catalogs;
+}
+
 =head2 GetQueryAndOptionList SELECTED_NODES
 
 Given an array reference of tree nodes that have been selected by the user,
@@ -209,10 +261,25 @@ sub __LinearizeTree {
         } else {
 
             my $clause = $node->getNodeValue;
-            $str .= $clause->{Key};
-            $str .= " ". $clause->{Op};
-            $str .= " ". $clause->{Value};
+            my $key = $clause->{Key};
+            $key .= "." . $clause->{Subkey} if defined $clause->{Subkey};
+            if ($key =~ s/(['\\])/\\$1/g or $key =~ /[^{}\w\.]/) {
+                $key = "'$key'";
+            }
+            my $value = $clause->{Value};
+            my $op = $clause->{Op};
+            if ( $value =~ /^NULL$/i && $op =~ /^(!?)=$/  ) {
+                $op = $1 ? 'IS NOT' : 'IS';
+            }
 
+            if ( $op =~ /^IS( NOT)?$/i ) {
+                $value = 'NULL';
+            } elsif ( $value !~ /^[+-]?[0-9]+$/ ) {
+                $value =~ s/(['\\])/\\$1/g;
+                $value = "'$value'";
+            }
+
+            $str .= $key ." ". $op . " " . $value;
         }
         $str =~ s/^\s+|\s+$//;
 
@@ -238,13 +305,14 @@ sub ParseSQL {
     my %args = (
         Query => '',
         CurrentUser => '', #XXX: Hack
+        Class => 'RT::Tickets',
         @_
     );
     my $string = $args{'Query'};
 
     my @results;
 
-    my %field = %{ RT::Tickets->new( $args{'CurrentUser'} )->FIELDS };
+    my %field = %{ $args{Class}->new( $args{'CurrentUser'} )->FIELDS };
     my %lcfield = map { ( lc($_) => $_ ) } keys %field;
 
     my $node =  $self;
@@ -258,30 +326,19 @@ sub ParseSQL {
     $callback{'Condition'} = sub {
         my ($key, $op, $value) = @_;
 
-        my ($main_key) = split /[.]/, $key;
+        my ($main_key, $subkey) = split /[.]/, $key, 2;
 
-        my $class;
-        if ( exists $lcfield{ lc $main_key } ) {
-            $key =~ s/^[^.]+/ $lcfield{ lc $main_key } /e;
-            ($main_key) = split /[.]/, $key;  # make the case right
-            $class = $field{ $main_key }->[0];
-        }
-        unless( $class ) {
+        unless( $lcfield{ lc $main_key} ) {
             push @results, [ $args{'CurrentUser'}->loc("Unknown field: [_1]", $key), -1 ]
         }
+        $main_key = $lcfield{ lc $main_key };
 
-        if ( lc $op eq 'is' || lc $op eq 'is not' ) {
-            $value = 'NULL'; # just fix possible mistakes here
-        } elsif ( $value !~ /^[+-]?[0-9]+$/ ) {
-            $value =~ s/(['\\])/\\$1/g;
-            $value = "'$value'";
-        }
+        # Hardcode value for IS / IS NOT
+        $value = 'NULL' if $op =~ /^IS( NOT)?$/i;
 
-        if ($key =~ s/(['\\])/\\$1/g or $key =~ /[^{}\w\.]/) {
-            $key = "'$key'";
-        }
-
-        my $clause = { Key => $key, Op => $op, Value => $value };
+        my $clause = { Key => $main_key, Subkey => $subkey,
+                       Meta => $field{ $main_key },
+                       Op => $op, Value => $value };
         $node->addChild( __PACKAGE__->new( $clause ) );
     };
     $callback{'Error'} = sub { push @results, @_ };
