@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -332,8 +332,9 @@ If this transaction has attached mime objects, returns the body of the first
 textual part (as defined in RT::I18N::IsTextualContentType).  Otherwise,
 returns the message "This transaction appears to have no content".
 
-Takes a paramhash.  If the $args{'Quote'} parameter is set, wraps this message 
-at $args{'Wrap'}.  $args{'Wrap'} defaults to 70.
+Takes a paramhash.  If the $args{'Quote'} parameter is set, wraps this message
+at $args{'Wrap'}.  $args{'Wrap'} is set from the $QuoteWrapWidth
+config variable.
 
 If $args{'Type'} is set to C<text/html>, this will return an HTML 
 part of the message, if available.  Otherwise it looks for a text/plain
@@ -348,7 +349,7 @@ sub Content {
     my %args = (
         Type => $PreferredContentType || '',
         Quote => 0,
-        Wrap  => 70,
+        Wrap => RT->Config->Get('QuoteWrapWidth') || 70,
         @_
     );
 
@@ -442,7 +443,9 @@ sub ApplyQuoteWrap {
         $max = length if length > $max;
     }
 
-    if ( $max > 76 ) {
+    # the addition of 6 here accounts for the extra characters added when quoting
+    # previously quoted text.
+    if ( $max > $args{cols} + 6 ) {
         require Text::Quoted;
         require Text::Wrapper;
 
@@ -1042,6 +1045,16 @@ sub _CanonicalizeRoleName {
                     $new = $date->AsString( Time => 0, Timezone => 'UTC' );
                 }
             }
+            elsif ( $cf->Type =~ /text/i) {
+                if (!defined($old) || ($old eq '')) {
+                    return ( "[_1] added", $field);   #loc()
+                }
+                if (!defined($new) || ($new eq '')) {
+                    return ( "[_1] deleted", $field);   #loc()
+                } else {
+                    return ( "[_1] changed", $field);   #loc()
+                }
+            }
         }
 
         if ( !defined($old) || $old eq '' ) {
@@ -1258,6 +1271,20 @@ sub _CanonicalizeRoleName {
                 }
             }
         }
+        elsif ( $self->Field =~ /Priority/ && RT->Config->Get('EnablePriorityAsString') ) {
+            my $object = $self->Object;
+            my ( $old_value, $new_value );
+            if ( $object->isa('RT::Ticket') ) {
+                $old_value = $object->_PriorityAsString( $self->OldValue );
+                $new_value = $object->_PriorityAsString( $self->NewValue );
+                $old_value = $self->loc($old_value) if $old_value;
+                $new_value = $self->loc($new_value) if $new_value;
+            }
+            $old_value //= $self->OldValue;
+            $new_value //= $self->NewValue;
+
+            return ( "[_1] changed from [_2] to [_3]", $self->loc( $self->Field ), "'$old_value'", "'$new_value'" );    #loc()
+        }
         else {
             return ( "[_1] changed from [_2] to [_3]",
                     $self->loc($self->Field),
@@ -1373,6 +1400,36 @@ sub _CanonicalizeRoleName {
         $principal->Load($self->Field);
         return ("Removed from group '[_1]'", $principal->Object->Name); #loc()
     },
+    Munge => sub {
+        my $self = shift;
+        return "Attachment content modified";
+    },
+    SetConfig => sub  {
+        my $self = shift;
+        my ($new_value, $old_value);
+
+        # pull in new value from reference if exists
+        if ( $self->NewReference ) {
+            my $newobj = RT::Configuration->new($self->CurrentUser);
+            $newobj->Load($self->NewReference);
+            $new_value = $newobj->Content;
+        }
+
+        # pull in old value from reference if exists
+        if ( $self->OldReference ) {
+            my $oldobj = RT::Configuration->new($self->CurrentUser);
+            $oldobj->Load($self->OldReference);
+            $old_value = $oldobj->Content;
+            return ('[_1] changed from "[_2]" to "[_3]"', $self->Field, $old_value // '', $new_value // ''); #loc()
+        }
+        else {
+            return ('[_1] changed to "[_2]"', $self->Field, $new_value // ''); #loc()
+        }
+    },
+    DeleteConfig => sub  {
+        my $self = shift;
+        return ('[_1] deleted"', $self->Field); #loc()
+    }
 );
 
 
@@ -1755,7 +1812,7 @@ sub ACLEquivalenceObjects {
 =head2 id
 
 Returns the current value of id.
-(In the database, id is stored as int(11).)
+(In the database, id is stored as int(19).)
 
 
 =cut
@@ -1982,7 +2039,7 @@ sub _CoreAccessible {
     {
 
         id =>
-                {read => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => ''},
+                {read => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(19)', default => ''},
         ObjectType =>
                 {read => 1, write => 1, sql_type => 12, length => 64,  is_blob => 0,  is_numeric => 0,  type => 'varchar(64)', default => ''},
         ObjectId =>
@@ -1992,7 +2049,7 @@ sub _CoreAccessible {
         Type =>
                 {read => 1, write => 1, sql_type => 12, length => 20,  is_blob => 0,  is_numeric => 0,  type => 'varchar(20)', default => ''},
         Field =>
-                {read => 1, write => 1, sql_type => 12, length => 40,  is_blob => 0,  is_numeric => 0,  type => 'varchar(40)', default => ''},
+                {read => 1, write => 1, sql_type => 12, length => 255,  is_blob => 0,  is_numeric => 0,  type => 'varchar(255)', default => ''},
         OldValue =>
                 {read => 1, write => 1, sql_type => 12, length => 255,  is_blob => 0,  is_numeric => 0,  type => 'varchar(255)', default => ''},
         NewValue =>
